@@ -49,7 +49,7 @@ namespace lp_api {
             m_constraints[1] = ct;
         }
 
-        virtual ~bound() {}
+        virtual ~bound() = default;
 
         theory_var get_var() const { return m_var; }
 
@@ -108,6 +108,7 @@ namespace lp_api {
         unsigned m_gomory_cuts;
         unsigned m_assume_eqs;
         unsigned m_branch;
+        unsigned m_bv_axioms;
         stats() { reset(); }
         void reset() {
             memset(this, 0, sizeof(*this));
@@ -128,6 +129,7 @@ namespace lp_api {
             st.update("arith-gomory-cuts", m_gomory_cuts);
             st.update("arith-assume-eqs", m_assume_eqs);
             st.update("arith-branch", m_branch);
+            st.update("arith-bv-axioms", m_bv_axioms);
         }
     };
 
