@@ -321,6 +321,13 @@ namespace smt {
         virtual bool is_shared(theory_var v) const {
             return false;
         }
+
+        /**
+           \brief Determine if node \c n under parent \c p is in a beta redex position.
+        */
+        virtual bool is_beta_redex(enode* p, enode* n) const {
+            return false;
+        }
     
         /**
            \brief Return true if the theory has something to propagate
@@ -361,7 +368,8 @@ namespace smt {
         //
         // ----------------------------------------------------
 
-        virtual void validate_model(model& mdl) {}
+        virtual void validate_model(proto_model& mdl) {}
+
 
         // ----------------------------------------------------
         //
@@ -615,6 +623,12 @@ namespace smt {
         bool is_relevant_and_shared(enode * n) const;
 
         bool assume_eq(enode * n1, enode * n2);
+
+
+        /**
+         * \brief theory plugin for fixed values.
+         */
+        virtual bool is_fixed_propagated(theory_var v, expr_ref& val, literal_vector & explain) { return false; }
     };
     
 };
